@@ -19,7 +19,7 @@ impl Column for SqliteColumn {
     }
 
     fn name(&self) -> &str {
-        &*self.name
+        &self.name
     }
 
     fn type_info(&self) -> &SqliteTypeInfo {
