@@ -4,10 +4,11 @@ use std::str::FromStr;
 /// Refer to [SQLite documentation] for the meaning of various synchronous settings.
 ///
 /// [SQLite documentation]: https://www.sqlite.org/pragma.html#pragma_synchronous
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Default)]
 pub enum SqliteSynchronous {
     Off,
     Normal,
+    #[default]
     Full,
     Extra,
 }
@@ -23,12 +24,6 @@ impl SqliteSynchronous {
     }
 }
 
-impl Default for SqliteSynchronous {
-    fn default() -> Self {
-        SqliteSynchronous::Full
-    }
-}
-
 impl FromStr for SqliteSynchronous {
     type Err = Error;
 
